<?php

namespace Odhen\API\Service;

use \SimpleXMLElement;

class Extratocons {

    protected $entityManager;
    protected $instanceManager;
    protected $sessionHandler;

    public function __construct(\Doctrine\ORM\EntityManager $entityManager, \Zeedhi\Framework\DependencyInjection\InstanceManager $instanceManager, \Helpers\Environment $sessionHandler){
        $this->entityManager = $entityManager;
        $this->instanceManager = $instanceManager;
        $this->sessionHandler = $sessionHandler;
    }

    public function consultaSaldo($CDFILIAL, $CDCLIENTE, $CDCONSUMIDOR, $CDFAMILISALD){
        $params = array(
            'P_TPOPERACAO' => 'CONSULTA_SALDO',
            'P_CDFILIAL' => $CDFILIAL,
            'P_CDCLIENTE' => $CDCLIENTE,
            'P_CDCONSUMIDOR' => $CDCONSUMIDOR,
            'P_CDFAMILISALD' => $CDFAMILISALD,
            'P_DTEXTRATO' => ''
        );
        $result = self::callExtratocons('RecuperaDados', $params);
        if (!empty($result) && !isset($result[0]['VRSALDCONFAM'])){
            $result = $result[0];
        }
        return $result;
    }

    public function consultaCartao($searchValue){
        $params = array(
            'P_TPOPERACAO' => 'CONSULTA_CARTAO',
            'P_CDFILIAL' => '',
            'P_CDCLIENTE' => '',
            'P_CDCONSUMIDOR' => $searchValue,
            'P_CDFAMILISALD' => '',
            'P_DTEXTRATO' => ''
        );
        $result = self::callExtratocons('RecuperaDados', $params);
        return $result;
    }

    public function consultaConsumer($searchValue, $CDFILIAL){
        $params = array(
            'P_TPOPERACAO' => 'CONSULTA_CONSUMIDOR_ONLINE',
            'P_CDFILIAL' => $CDFILIAL,
            'P_CDCLIENTE' => '',
            'P_CDCONSUMIDOR' => $searchValue,
            'P_TCDCLIENTE' => 'T',
            'P_IDSITCONSUMI' => 'T',
            'P_CDFAMILISALD' => '',
            'P_DTEXTRATO' => ''
        );
        $result = self::callExtratocons('RecuperaDados', $params);
        return $result;
    }

    public function consultaConsumoFamilia($CDCLIENTE, $CDCONSUMIDOR, $date){
        $params = array(
            'P_TPOPERACAO' => 'CONSULTA_CONSUMO_FAMILIA',
            'P_CDFILIAL' => '',
            'P_CDCLIENTE' => $CDCLIENTE,
            'P_CDCONSUMIDOR' => $CDCONSUMIDOR,
            'P_CDFAMILISALD' => '',
            'P_DTEXTRATO' => $date
        );
        $result = self::callExtratocons('RecuperaDados', $params);
        return $result;
    }

    public function insereExtratocons($params){
        $xml = new SimpleXMLElement('<ExtratoCons><Registro></Registro></ExtratoCons>');
        foreach ($params as $key => $value){
            $xml->Registro->addChild(strtolower($key), $value);
        }

        $params = array(
            'P_XMLEXTCONS' => $xml->asXml()
        );
        self::callExtratocons('InsereDados', $params);
    }

    public function consultaFidelidade($CPF){
        $params = array( "cpf" => $CPF );

        $result = self::callFidelity('user', $params);
        if(isset($result['bloqueado'])){
            if($result['bloqueado']){
                throw new \Exception("Usuário encontra-se bloqueado: <br>".$result['message']);
            }
        }
        if (!$result['success']){
            $result['userInfo'] = array(
                'cpf' => $CPF,
                'balance' => 0,
                'fidelidade' => $FIDELIDADE
            );
        }
        return $result;
    }

    public function validaCupom($storeCode, $CPF4ALL, $CODPROMO, $SUBTOTAL, $productString){
        $params = array(
            "userId" => $CPF4ALL,
            "friendlyCode" => $CODPROMO,
            "storeReference" => $storeCode,
            "channel" => "pdv",
            "products" => $productString,
            "payment" => array(
                "subTotal" => $SUBTOTAL
            )
        );

        $result = self::callFidelity('validate', json_encode($params));

        if (!$result['success']) throw new \Exception("<br>".$result['message']);
        else return $result;
    }

    public function transmiteVenda($vendaJSON){
        $result = self::callFidelity('order', $vendaJSON);
        if (!$result['success']) throw new \Exception("<br>".$result['message']);
        else return $result;
    }
    
    public function cancelaFidelidade($orderId){
       	return self::cancela4ALL($orderId);
    }

    public function cancela4ALL($orderId){
        $params = array(
            "orderId" => $orderId
        );
        $result = $this::callFidelity('order/cancel', json_encode($params));

        if (!$result['success']) throw new \Exception("<br>".$result['message']);
        else return $result;
    }

    private function callExtratocons($method, $params){
        self::LogExtratoCons('');
        self::LogExtratoCons('callExtratocons');
        $URL = $this->sessionHandler->getUserInfo()['CDURLWSEXTCONS'];
        $URL = rtrim($URL, '/') . "/" .$method;
        self::LogExtratoCons($URL);
        $postfields = "";
        foreach ($params as $key => $param){
            $postfields .= "&".$key."=".$param;
        }
        $postfields = substr($postfields, 1);
        self::LogExtratoCons('PARAMETERS: '.$postfields);

        $curl = curl_init();

        curl_setopt($curl, CURLOPT_URL, $URL);
        curl_setopt($curl, CURLOPT_CUSTOMREQUEST, 'POST');
        curl_setopt($curl, CURLOPT_HTTPHEADER, array('Content-Type: application/x-www-form-urlencoded'));
        curl_setopt($curl, CURLOPT_POSTFIELDS, $postfields);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($curl, CURLOPT_ENCODING,'');
        curl_setopt($curl, CURLOPT_MAXREDIRS,10);
        curl_setopt($curl, CURLOPT_FOLLOWLOCATION, true);
        curl_setopt($curl, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);
        curl_setopt($curl, CURLOPT_AUTOREFERER, true);
        curl_setopt($curl, CURLOPT_POSTREDIR, CURL_REDIR_POST_ALL);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_TIMEOUT, 30);
        curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, 30);

        if (!empty($this->instanceManager->getParameter('PROXY_IP'))){
            self::LogExtratoCons("PROXY_IP: ".$this->instanceManager->getParameter('PROXY_IP').":".$this->instanceManager->getParameter('PROXY_PORT'));
            curl_setopt($curl, CURLOPT_PROXY, $this->instanceManager->getParameter('PROXY_IP').":".$this->instanceManager->getParameter('PROXY_PORT'));
            if (!empty($this->instanceManager->getParameter('PROXY_USER'))){
                self::LogExtratoCons("PROXY_USER: ".$this->instanceManager->getParameter('PROXY_USER').":".$this->instanceManager->getParameter('PROXY_PASSWORD'));
                curl_setopt($curl, CURLOPT_PROXYUSERPWD, $this->instanceManager->getParameter('PROXY_USER').":".$this->instanceManager->getParameter('PROXY_PASSWORD'));
            }
        }
        $curl_response = curl_exec($curl);
        self::LogExtratoCons('RESPONSE: '.$curl_response);
        $err    = curl_errno($curl);
        $errmsg = curl_error($curl);
        $httpcode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
        self::LogExtratoCons('httpcode: '.$httpcode);
        curl_close($curl);

        if ($httpcode ==='404' || $httpcode === 404){
            self::LogExtratoCons('Erro 404 Servidor não encontrado.');
            throw new \Exception("<br>Erro 404 Servidor não encontrado.");
        }
        if ($err){
            self::LogExtratoCons($err);
            self::LogExtratoCons($errmsg);
            if ($err === 28){
                throw new \Exception("<br>N&atilde;o foi poss&iacute;vel estabelecer conex&atilde;o com o servidor do EXTRATOCONS ONLINE.");
            } else {
                throw new \Exception("<br>Ocorreu um erro na chamada do EXTRATOCONS ONLINE:<br><br>" . $errmsg);
            }
        }

        if ($method == 'RecuperaDados'){
            if (!self::isXml($curl_response)){
                throw new \Exception("<br>O EXTRATOCONS ONLINE retornou uma mensagem inesperada:<br><br>" . $curl_response);
            } else {
                try {
                $xml = simplexml_load_string($curl_response);
                } catch (\Exception $e) {
                    throw new \Exception("<br>O servidor EXTRATOCONS ONLINE retornou um xml ilegível" . $curl_response);
                }
                $json_result = json_encode($xml);

                $index = 0;
                while (strpos($json_result, 'Registro') > -1){
                    $json_result = preg_replace(preg_quote('"Registro"', '/'), $index++, $json_result, 1);
                }

                $result = json_decode($json_result, true);

                if(!isset($result['ExtratoCons'])){
                    $result['ExtratoCons'] = $result;
                }

                return $result['ExtratoCons'];
            }
        } else if ($method == 'InsereDados' && $curl_response !== "OK"){
            throw new \Exception("<br>O EXTRATOCONS ONLINE retornou um erro:<br><br>" . $curl_response);
        }
    }

    private function callFidelity($method, $params){
        $URL = $this->sessionHandler->getUserInfo()['CDURLWSFID4ALL'];
        $URL = rtrim($URL, '/') . "/";
        $URL .= $method;
        if ($method === 'user') $URL .= '?'.http_build_query($params);

        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, $URL);

        if ($method === 'order' || $method === 'validate'){
            curl_setopt($curl, CURLOPT_HTTPHEADER, array('Content-Type:application/json'));
            curl_setopt($curl, CURLOPT_POSTFIELDS, $params);
        }
        if ($method === 'order/cancel'){
            curl_setopt($curl, CURLOPT_CUSTOMREQUEST, 'PUT');
            curl_setopt($curl, CURLOPT_HTTPHEADER, array('Content-Type:application/json'));
            curl_setopt($curl, CURLOPT_POSTFIELDS, $params);
        }

        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_TIMEOUT, 20);
        curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, 10);

        $curl_response = curl_exec($curl);
        $err    = curl_errno($curl);
        $errmsg = curl_error($curl);
        curl_close($curl);

        if ($err){
            if ($err === 28){
                throw new \Exception("<br>N&atilde;o foi poss&iacute;vel estabelecer uma conex&atilde;o com o servidor.");
            } else {
                throw new \Exception("<br>Ocorreu um erro na chamada com o servidor:<br><br>" . $errmsg);
            }
        }
        if (empty($curl_response)){
            throw new \Exception("<br>N&atilde;o houve retorno do servidor.");
        }
        $result = json_decode($curl_response, true);
        if (empty($result)) throw new \Exception($curl_response);
        $result['fidelidade']='4ALL';
        if (array_key_exists('success', $result)){
            if ($method === 'user' || $result['success']){
                return $result;
            } else {
                throw new \Exception($result['message']);
            }
        } else {
            throw new \Exception($result['error']);
        }
    }

    public function isXml($string){
        return substr($string, 0, 5) == "<?xml";
    }

    public function LogExtratoCons($texto){
        return self::Log('ExtratoCons',$texto);
    }

    public function Log($tipo,$texto){
        if($tipo=='ExtratoCons'){
            $tipo ='LogExtratoCons';
        }
        $paths = ['LOG/', $tipo.'/', (new \DateTime())->format('m-Y')];
        $logVendaPath = dirname(__DIR__) . '/../../../../../../';
        foreach ($paths as $path){
            $logVendaPath = $logVendaPath . $path;
            if (!is_dir($logVendaPath)){
                mkdir($logVendaPath, 0700);
            }
        }
        $logVendaPath = $logVendaPath .'/'. (new \DateTime())->format('d-m-Y') . '.txt';
        $horaAtual = (new \DateTime())->format('d/m/Y H:i:s');
        file_put_contents($logVendaPath, $horaAtual . ' - ' . $texto . "\r\n", FILE_APPEND);
    }
}